@@ -508,12 +508,13 @@ char *argv[];
 	exit(exit_stat);
 }
 
+/* This saves the "envelop sender", aka "From " line */
 void save_from(buffer)
 char *buffer;
 {
 	/** save the SECOND word of this string as FROM **/
 
-	register char *f = from;
+	register char *f = Efrom;
 
 	while (!whitespace(*buffer))
 	  buffer++;				/* get to word     */
@@ -527,6 +528,120 @@ char *buffer;
 	*f = '\0';				/* Null terminate! */
 }
 
+
+/* This is a utility function for save_subject().
+ * If an encoded subject line (eg iso-8859-1) is encountered,
+ * it is up to us to decode it.
+ * we assume that the first two chars are =?, as in
+ *  =?ISO-8859-1?B?xxxxxxxxx?=
+ */
+void iso_convert_sub(char *subject)
+{
+	static char *base=
+	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+	unsigned char b1,b2,b3,b4;
+
+	char *ptr, *src,*dest, *end;
+	char *start=strstr(subject, "=?");
+	char tmpsubj[LONG_STRING];
+	int chunklen=0;
+	int enc_type=0; /*1=quoted-printable, 2=binary */
+	char enc_bytes[2];
+
+	/* There are two kinds of encoding: "quoted-prinable", and "binary".
+	 * substitute single char for "quoted-printable",
+	 * so that single x-out matching can happen. eg:
+	 *  /p.nis/
+	 */
+
+	if(start==NULL) return;
+	ptr=strchr(&start[2],'?');
+
+	switch(ptr[1]){
+	    case 'Q':
+	    case 'q':
+		enc_type=1;
+		break;
+
+	    case 'B':
+	    case 'b':
+		enc_type=2;
+		break;
+
+	    default:
+		return;
+	}
+
+	ptr+=2;
+	if(*ptr != '?'){
+		fprintf(stderr,
+		 "DEBUG: fractured encoded subj found in iso_convert_sub\n");
+		return;
+	}
+	if(strchr(&ptr[1],'?')==NULL){
+		fprintf(stderr,
+		 "DEBUG: truncated encoded subj found in iso_convert_sub\n");
+		return;
+	}
+	ptr++;
+
+	/* hack up quoted-printable, then get outta here*/
+	if(enc_type == 1){
+		src=ptr;
+		dest=tmpsubj;
+
+		while(*src != '?'){
+			if(*src == '='){
+				src++;
+				switch(*src){
+					case 'E':
+						*dest='e';
+						break;
+					case 'F':
+						*dest='n';
+						break;
+					default:
+						*dest='X';
+				}
+			} else {
+				*dest = *src;
+			}
+
+			dest++; src++;
+		}
+		*dest='\0';
+		stringcopy(subject, tmpsubj, dest+1 -tmpsubj);
+		return;
+	}
+
+	/* else, enc_type==2 : binary encoding, base64 */
+	/* ptr == where start of encoded data is, in caller's buffer */
+	end=strstr(ptr, "?=");
+	dest=tmpsubj;
+
+	if(end==NULL){
+		fprintf(stderr,
+		 "DEBUG: truncated encoded subj found in iso_convert_sub\n");
+		return;
+	}
+	while(ptr<end){
+		b1=strchr(base, ptr[0]) - base;
+		b2=strchr(base, ptr[1]) - base;
+		b3=strchr(base, ptr[2]) - base;
+		b4=strchr(base, ptr[3]) - base;
+		ptr+=4;
+		dest[0] = (b1<<2) | ((b2>>4)&0xff);
+		dest[1] = (b2 <<4 ) | ((b3>>2)&0xff);
+		dest[2] = (b3<<6)&0xff | (b4 & 0xff);
+		dest +=3;
+	}
+	end+=2;
+	stringcopy(dest, end, strlen(end)+1);
+	stringcopy(start, tmpsubj, strlen(tmpsubj)+1);
+	
+}
+
+
 /* reasonably assume buffer is at least SLEN long */
 void save_subject(buffer)
 char *buffer;
@@ -534,11 +649,29 @@ char *buffer;
 	/** save all but the word "Subject:" for the subject **/
 
 	register int skip = 8;  /* skip "Subject:" initially */
+	register int frompos, topos;
 
 	while (whitespace(buffer[skip]))
 		skip++;
 
 	stringcopy(subject, (char *) buffer + skip, LONG_STRING);
+	if(strncmp(subject, "=?", 2)==0){
+		iso_convert_sub(subject);
+	}
+
+	/* now make an ascii-only copy of subject line, to de-mangle
+	 * funky subject lines.
+	 * Furthermore, force to lowercase, so regex's on it dont have
+	 * to get all funky.
+	 */
+	topos=0;
+	for(frompos=0; frompos<LONG_STRING, subject[frompos]!='\0'; frompos++){
+		if(isalpha(subject[frompos])){
+			alphasubject[topos++]=(tolower(subject[frompos]));
+		}
+	}
+	alphasubject[topos]='\0';
+	
 }
 
 void save_sender(buffer)
@@ -652,19 +785,22 @@ char *buffer, *fieldname;
 	register int i, j = 0;
 
 	/** first let's extract the address from this line.. **/
+	/* Note that if buffer is obnoxiously long, we may truncate*/
 
 	if (buffer[strlen(buffer)-1] == '>') {	/* case #1 */
 	  for (i=strlen(buffer)-1; buffer[i] != '<' && i > 0; i--)
 		/* nothing - just move backwards .. */ ;
 	  i++;	/* skip the leading '<' symbol */
-	  while (buffer[i] != '>')
+	  while ((buffer[i] != '>') && (j < (LONG_STRING-1) )){
 	    address[j++] = buffer[i++];
+	  }
 	  address[j] = '\0';
 	}
 	else {	/* get past "from:" and copy until white space or paren hit */
 	  for (i=strlen(fieldname); whitespace(buffer[i]); i++)
 	     /* skip past that... */ ;
-	  while (buffer[i] != '(' && ! whitespace(buffer[i]) && buffer[i]!='\0')
+	  while (buffer[i] != '(' && ! whitespace(buffer[i]) &&
+	         buffer[i]!='\0' && (j < (LONG_STRING-1) ))
 	    address[j++] = buffer[i++];
 	  address[j] = '\0';
 	}
@@ -672,13 +808,17 @@ char *buffer, *fieldname;
 	/** now let's see if we should overwrite the existing from address
 	    with this one or not.. **/
 
-	if (processed_a_reply_to)
-	  return;	/* forget it! */
 
-	stringcopy(from, address, LONG_STRING);	/* replaced!! */
 
-	if (istrcmp(fieldname, "Reply-To:") == 0)
+	if (istrcmp(fieldname, "Reply-To:") == 0){
+	  if (processed_a_reply_to)
+	    return;	/* forget it! */
 	  processed_a_reply_to++;
+	  stringcopy(replyto, address, LONG_STRING);	/* replaced!! */
+	} else {
+	  /* Assume "From:"  header */
+	  stringcopy(Hfrom, address, LONG_STRING);
+	}
 }
 #endif
 
