@@ -180,9 +180,10 @@ struct condition_rec *cond;
 
 	rule_lines = atoi(cond->argument1);
 
+#ifdef DEBUG
 	fprintf(stderr,"DEBUG: compare: line=%d, rule_lines=%d\n",
 		line, rule_lines);
-
+#endif
 
 	switch (relop) {
 	  case LE: return(line <= rule_lines);
@@ -221,10 +222,26 @@ action_from_ruleset()
 	      case RCPT_TO: x = matches(rcpt_to, relation, cond);	break;
 	      case TO     : x = multimatch(tolist, relation, cond);	break;
 	      case MAILLIST:x = multimatch(mail_lists, relation, cond);	break;
-	      case FROM   : x = matches(from, relation, cond); 		break;
 	      case SENDER : x = matches(sender, relation, cond);	break;
 	      case SUBJECT: x = matches(subject, relation, cond);	break;
+	      case ALPHASUBJECT: x=matches(alphasubject, relation, cond);break;
 	      case LINES  : x = compare(lines, relation, cond);		break;
+	      /* multi-match: any of the "from" type headers will
+	       * trigger a match
+	       */
+	      case FROM:
+/* DEBUG*/
+		printf("Hfrom: %d\n", matches(Hfrom, relation, cond));
+		printf("Efrom: %d\n", matches(Efrom, relation, cond));
+		printf("sender: %d\n", matches(sender, relation, cond));
+		printf("replyto: %d\n", matches(replyto, relation, cond));
+/**/
+
+ x = (matches(Hfrom, relation, cond) ||
+	                      matches(Efrom, relation, cond) ||
+	                      matches(sender, relation, cond) ||
+	                      matches(replyto, relation, cond));
+	          break;
 		       
 	      case CONTAINS: if (outfptr != NULL) fprintf(outfptr,
 				catgets(elm_msg_cat,
@@ -335,7 +352,7 @@ int  line, display;
 						 "<return-address>"),
 				   SLEN);
 			 else
-			   strncat(buffer, from, SLEN);
+			   strncat(buffer, Hfrom, SLEN);
 	                 bufflen = strlen(buffer);
 			 break;
 
@@ -570,6 +587,7 @@ int n;
 	  case MAILLIST     : return("mailinglist");
 	  case SENDER : return("sender");
 	  case SUBJECT: return("subject");
+	  case ALPHASUBJECT: return("alphasubject");
 	  case LINES  : return ("lines");
 	  case CONTAINS: return("contains");
 	  default     : sprintf(buffer, "?%d?", n); return((char *)buffer);
